@@ -37,12 +37,12 @@ import sys
 import typing
 import urllib.parse
 import webbrowser
+from pathlib import Path
 
 import blinker
-import pkg_resources
 
 from nikola.plugin_categories import Command
-from nikola.utils import dns_sd, req_missing, get_theme_path, makedirs
+from nikola.utils import dns_sd, req_missing, get_theme_path, makedirs, pkg_resources_path
 
 try:
     import aiohttp
@@ -57,16 +57,15 @@ except ImportError:
 
 try:
     from watchdog.observers import Observer
+    from watchdog.observers.polling import PollingObserver
 except ImportError:
     Observer = None
+    PollingObserver = None
 
 LRJS_PATH = os.path.join(os.path.dirname(__file__), 'livereload.js')
 REBUILDING_REFRESH_DELAY = 0.35
 IDLE_REFRESH_DELAY = 0.05
 
-if sys.platform == 'win32':
-    asyncio.set_event_loop(asyncio.ProactorEventLoop())
-
 
 def base_path_from_siteuri(siteuri: str) -> str:
     """Extract the path part from a URI such as site['SITE_URL'].
@@ -162,6 +161,16 @@ class CommandAuto(Command):
             'type': str,
             'help': "Database backend ('dbm', 'json', 'sqlite3')",
             'section': 'Arguments passed to `nikola build`'
+        },
+        {
+            # We might be able to improve on this
+            # if and when https://github.com/gorakhargosh/watchdog/issues/365
+            # is ever fixed.
+            'name': 'poll',
+            'long': 'poll',
+            'default': False,
+            'type': bool,
+            'help': 'Use polling to notice changes behind symbolic links. This may reduce performance.'
         }
     ]
 
@@ -223,7 +232,7 @@ class CommandAuto(Command):
             watched.add(item)
         watched |= self.site.registered_auto_watched_folders
         # Nikola itself (useful for developers)
-        watched.add(pkg_resources.resource_filename('nikola', ''))
+        watched.add(pkg_resources_path('nikola', ''))
 
         out_folder = self.site.config['OUTPUT_FOLDER']
         if not os.path.exists(out_folder):
@@ -259,7 +268,7 @@ class CommandAuto(Command):
         # Run an initial build so we are up-to-date. The server is running, but we are not watching yet.
         loop.run_until_complete(self.run_initial_rebuild())
 
-        self.wd_observer = Observer()
+        self.wd_observer = Observer() if not options['poll'] else PollingObserver()
         # Watch output folders and trigger reloads
         if self.has_server:
             self.wd_observer.schedule(NikolaEventHandler(self.reload_page, loop), out_folder, recursive=True)
@@ -275,27 +284,18 @@ class CommandAuto(Command):
         self.wd_observer.schedule(ConfigEventHandler(_conf_fn, self.queue_rebuild, loop), _conf_dn, recursive=False)
         self.wd_observer.start()
 
-        win_sleeper = None
-        # https://bugs.python.org/issue23057 (fixed in Python 3.8)
-        if sys.platform == 'win32' and sys.version_info < (3, 8):
-            win_sleeper = asyncio.ensure_future(windows_ctrlc_workaround())
-
         if not self.has_server:
             self.logger.info("Watching for changes...")
             # Run the event loop forever (no server mode).
             try:
                 # Run rebuild queue
                 loop.run_until_complete(self.run_rebuild_queue())
-
                 loop.run_forever()
             except KeyboardInterrupt:
                 pass
             finally:
-                if win_sleeper:
-                    win_sleeper.cancel()
                 self.wd_observer.stop()
                 self.wd_observer.join()
-            loop.close()
             return
 
         if options['ipv6'] or '::' in host:
@@ -326,16 +326,17 @@ class CommandAuto(Command):
             pass
         finally:
             self.logger.info("Server is shutting down.")
-            if win_sleeper:
-                win_sleeper.cancel()
             if self.dns_sd:
                 self.dns_sd.Reset()
             rebuild_queue_fut.cancel()
             reload_queue_fut.cancel()
+
+            # Not sure why this isn't done by the web_runner.cleanup() code:
+            loop.run_until_complete(self.remove_websockets(None))
+
             loop.run_until_complete(self.web_runner.cleanup())
             self.wd_observer.stop()
             self.wd_observer.join()
-        loop.close()
 
     async def set_up_server(self, host: str, port: int, base_path: str, out_folder: str) -> None:
         """Set up aiohttp server and start it."""
@@ -482,7 +483,7 @@ class CommandAuto(Command):
 
         return ws
 
-    async def remove_websockets(self, app) -> None:
+    async def remove_websockets(self, _app) -> None:
         """Remove all websockets."""
         for ws in self.sockets:
             await ws.close()
@@ -512,13 +513,6 @@ class CommandAuto(Command):
             self.sockets.remove(ws)
 
 
-async def windows_ctrlc_workaround() -> None:
-    """Work around bpo-23057."""
-    # https://bugs.python.org/issue23057
-    while True:
-        await asyncio.sleep(1)
-
-
 class IndexHtmlStaticResource(StaticResource):
     """A StaticResource implementation that serves /index.html in directory roots."""
 
@@ -539,8 +533,13 @@ class IndexHtmlStaticResource(StaticResource):
     async def handle_file(self, request: 'web.Request', filename: str, from_index=None) -> 'web.Response':
         """Handle file requests."""
         try:
-            filepath = self._directory.joinpath(filename).resolve()
-            if not self._follow_symlinks:
+            unresolved_path = self._directory.joinpath(filename)
+            if self._follow_symlinks:
+                normalized_path = Path(os.path.normpath(unresolved_path))
+                normalized_path.relative_to(self._directory)
+                filepath = normalized_path.resolve()
+            else:
+                filepath = unresolved_path.resolve()
                 filepath.relative_to(self._directory)
         except (ValueError, FileNotFoundError) as error:
             # relatively safe
@@ -621,5 +620,5 @@ class ConfigEventHandler(NikolaEventHandler):
 
     def dispatch(self, event):
         """Handle file events if they concern the configuration file."""
-        if event._src_path == self.configuration_filename:
+        if event.src_path == self.configuration_filename:
             super().dispatch(event)
